#include "stdafx.h"
#include "Windows.h"
#include <stdlib.h>

#define SERVICE_NAME "tcsd"

SERVICE_STATUS_HANDLE WINAPI serviceStatusHandle;
HANDLE killServiceEvent=INVALID_HANDLE_VALUE;
BOOL StartServiceThread();
VOID ServiceCtrlHandler (DWORD controlCode);
void KillService(void);
BOOL serviceRunning = 0;
BOOL servicePaused = 0;
HANDLE threadHandle;
DWORD serviceCurrentStatus;

VOID ServiceMain(DWORD argc, char *argv[]);
int UpdateSCMStatus (DWORD dwCurrentState, DWORD dwWin32ExitCode,
					 DWORD dwServiceSpecificExitCode, DWORD dwCheckPoint,
					 DWORD dwWaitHint);
void terminateService(int iErr);
DWORD ServiceExecutionThread(LPDWORD param);


int _tmain(int argc, _TCHAR* argv[])
{
	SERVICE_TABLE_ENTRY serviceTable[] =
	{
		{(LPWSTR)SERVICE_NAME, (LPSERVICE_MAIN_FUNCTION) ServiceMain},
		{ NULL, NULL }
	};

	// registrazione del servizio 
	BOOL success = StartServiceCtrlDispatcher(serviceTable);
	if (!success)
	{
		return 1;
	}
	return 0;
}


VOID ServiceMain(DWORD argc, char* argv[])
{
	// registrazione della funzione che gestisce i comandi dellSCM
	serviceStatusHandle = RegisterServiceCtrlHandler((LPWSTR)SERVICE_NAME,
		(LPHANDLER_FUNCTION) ServiceCtrlHandler);

	if (!serviceStatusHandle)
	{
		terminateService(GetLastError());
		return;
	}
	// Notifico al SCM che la richiesta di attivare il servizio e pendente
	BOOL success = UpdateSCMStatus(SERVICE_START_PENDING, 
		NO_ERROR, 0, 1, 5000);
	if (!success) {
		terminateService(GetLastError());
		return;
	}
	// creo un evento che verra utilizzato per terminare il servizio
	killServiceEvent = CreateEvent (0, TRUE, FALSE, 0);
	if (!killServiceEvent) {
		terminateService(GetLastError());
		return;
	}
	// Notifico al SCM che la richiesta di attivare il servizio e pendente
	success = UpdateSCMStatus(SERVICE_START_PENDING, 
		NO_ERROR, 0, 2, 1000);
	if (!success) {
		terminateService(GetLastError());
		return;
	}


	// Creo il thread che implementa il servizio
	success = StartServiceThread();

	if (!success){
		terminateService(GetLastError());
		return;
	}
	// Il servizio e attivo. Lo notifico allSCM
	serviceCurrentStatus = SERVICE_RUNNING;
	success = UpdateSCMStatus(SERVICE_RUNNING, NO_ERROR, 0, 0, 0);
	if (!success){
		terminateService(GetLastError());
		return;
	}
	//attendo il segnale di terminazione del thread
	WaitForSingleObject (killServiceEvent, INFINITE);

	//Segnalo che il servizio si sta arrestando
	Beep(800,2000);
	Sleep(2000);

	CloseHandle(killServiceEvent);
	killServiceEvent = INVALID_HANDLE_VALUE;
	terminateService(0);
}


DWORD ServiceExecutionThread(LPDWORD param)
{
	while (serviceRunning)
	{
		Beep(200,200);
		Sleep(beepDelay);
	}
	return 0;
}

BOOL StartServiceThread()
{
	DWORD id;
	// Creazione del thread
	threadHandle = CreateThread(0, 0,
		(LPTHREAD_START_ROUTINE) ServiceExecutionThread,
		0, 0, &id);
	if (threadHandle == 0) {
		return FALSE;
	}
	else {
		serviceRunning = TRUE;
		return TRUE;
	}
}

VOID ServiceCtrlHandler (DWORD controlCode)
{
	BOOL success;
	switch(controlCode)
	{

	case SERVICE_CONTROL_PAUSE:
		if (serviceRunning && !servicePaused)
		{
			success = UpdateSCMStatus(SERVICE_PAUSE_PENDING, NO_ERROR,
				0, 1, 1000);
			servicePaused = TRUE;
			SuspendThread(threadHandle);
			serviceCurrentStatus = SERVICE_PAUSED;
		}
		break;

	case SERVICE_CONTROL_CONTINUE:
		if (serviceRunning && servicePaused)
		{
			success = UpdateSCMStatus(SERVICE_CONTINUE_PENDING, NO_ERROR, 0,
				1, 1000);
			servicePaused=FALSE;
			ResumeThread(threadHandle);
			serviceCurrentStatus = SERVICE_RUNNING;
		}
		break;

	case JW_SERVICE_CONTROL_BEEP_NOW:
		Beep(800, 800);
		break;

	case SERVICE_CONTROL_INTERROGATE:
		break;

	case SERVICE_CONTROL_SHUTDOWN:

	case SERVICE_CONTROL_STOP:

		serviceCurrentStatus = SERVICE_STOP_PENDING;
		success = UpdateSCMStatus(SERVICE_STOP_PENDING, NO_ERROR, 0, 1, 5000);
		terminateService(0);
		return;
	default:
		break;
	}
	UpdateSCMStatus(serviceCurrentStatus, NO_ERROR, 0, 0, 0);
}

// Questa funzione aggiorna lo stato del servizio nell'SCM
BOOL UpdateSCMStatus (DWORD dwCurrentState,
					  DWORD dwWin32ExitCode,
					  DWORD dwServiceSpecificExitCode,
					  DWORD dwCheckPoint,
					  DWORD dwWaitHint)
{
	BOOL success;
	SERVICE_STATUS serviceStatus;

	serviceStatus.dwServiceType = SERVICE_WIN32_OWN_PROCESS;
	serviceStatus.dwCurrentState = dwCurrentState;

	if (dwCurrentState == SERVICE_START_PENDING)
	{
		serviceStatus.dwControlsAccepted = 0;
	}
	else
	{
		serviceStatus.dwControlsAccepted =
			SERVICE_ACCEPT_STOP |
			SERVICE_ACCEPT_PAUSE_CONTINUE |
			SERVICE_ACCEPT_SHUTDOWN;
	}

	if (dwServiceSpecificExitCode == 0)
	{
		serviceStatus.dwWin32ExitCode = dwWin32ExitCode;
	}
	else
	{
		serviceStatus.dwWin32ExitCode = ERROR_SERVICE_SPECIFIC_ERROR;
	}
	serviceStatus.dwServiceSpecificExitCode =   dwServiceSpecificExitCode;
	serviceStatus.dwCheckPoint = dwCheckPoint;
	serviceStatus.dwWaitHint = dwWaitHint;

	success = SetServiceStatus (serviceStatusHandle, &serviceStatus);
	if (!success)
	{
		terminateService(0);
	}
	return success;
}


void terminateService(int iErr)
{
	serviceRunning = 0;
	if(killServiceEvent!=INVALID_HANDLE_VALUE)
		SetEvent(killServiceEvent);
	UpdateSCMStatus(SERVICE_STOPPED,iErr,0,0,0);
}